@@ -13,6 +13,9 @@
 #define __MIDORI_BROWSER_H__
 
 #include <webkit/webkit.h>
+#if defined(HAVE_HILDON) && HAVE_HILDON
+    #include <hildon/hildon.h>
+#endif
 
 #include <katze/katze.h>
 
@@ -36,7 +39,11 @@ typedef struct _MidoriBrowserClass           MidoriBrowserClass;
 
 struct _MidoriBrowserClass
 {
+    #if defined(HAVE_HILDON) && HAVE_HILDON
+    HildonWindowClass parent_class;
+    #else
     GtkWindowClass parent_class;
+    #endif
 
     /* Signals */
     void
@@ -93,6 +100,10 @@ void
 midori_browser_activate_action        (MidoriBrowser*     browser,
                                        const gchar*       name);
 
+void
+midori_browser_set_current_uri        (MidoriBrowser*     browser,
+                                       const gchar*       uri);
+
 const gchar*
 midori_browser_get_current_uri        (MidoriBrowser*     browser);
 
