@@ -28,6 +28,7 @@ struct _MidoriLocationAction
 
     gchar* uri;
     gdouble progress;
+    gchar* secondary_icon;
 
     GtkTreeModel* model;
     GtkTreeModel* filter_model;
@@ -82,6 +83,12 @@ midori_location_action_set_property (GObject*      object,
                                      const GValue* value,
                                      GParamSpec*   pspec);
 
+static void
+midori_location_action_get_property (GObject*    object,
+                                     guint       prop_id,
+                                     GValue*     value,
+                                     GParamSpec* pspec);
+
 static void
 midori_location_action_activate (GtkAction* object);
 
@@ -154,6 +161,7 @@ midori_location_action_class_init (MidoriLocationActionClass* class)
     gobject_class = G_OBJECT_CLASS (class);
     gobject_class->finalize = midori_location_action_finalize;
     gobject_class->set_property = midori_location_action_set_property;
+    gobject_class->get_property = midori_location_action_get_property;
 
     action_class = GTK_ACTION_CLASS (class);
     action_class->activate = midori_location_action_activate;
@@ -168,7 +176,7 @@ midori_location_action_class_init (MidoriLocationActionClass* class)
                                      "Progress",
                                      "The current progress of the action",
                                      0.0, 1.0, 0.0,
-                                     G_PARAM_WRITABLE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_SECONDARY_ICON,
@@ -177,7 +185,7 @@ midori_location_action_class_init (MidoriLocationActionClass* class)
                                      "Secondary",
                                      "The stock ID of the secondary icon",
                                      NULL,
-                                     G_PARAM_WRITABLE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -233,6 +241,9 @@ midori_location_action_freeze (MidoriLocationAction* location_action)
     katze_object_assign (location_action->sort_model, NULL);
     katze_object_assign (location_action->filter_model, NULL);
     midori_location_action_set_model (location_action, NULL);
+
+    if (location_action->items)
+        g_hash_table_remove_all (location_action->items);
 }
 
 /**
@@ -248,6 +259,9 @@ midori_location_action_thaw (MidoriLocationAction* location_action)
 {
     GtkTreeModel* sort_model;
     GtkTreeModel* filter_model;
+    GtkTreeIter iter;
+    GtkTreeIter child_iter;
+    gint i;
 
     g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
     g_return_if_fail (midori_location_action_is_frozen (location_action));
@@ -263,6 +277,15 @@ midori_location_action_thaw (MidoriLocationAction* location_action)
 
     location_action->filter_model = filter_model;
     location_action->sort_model = sort_model;
+
+    i = MAX_ITEMS;
+    while (gtk_tree_model_iter_nth_child (sort_model, &iter, NULL, i++))
+    {
+        gtk_tree_model_sort_convert_iter_to_child_iter (
+            GTK_TREE_MODEL_SORT (sort_model), &child_iter, &iter);
+        gtk_list_store_set (GTK_LIST_STORE (location_action->model),
+                    &child_iter, VISIBLE_COL, FALSE, -1);
+    }
 }
 
 static void
@@ -270,6 +293,7 @@ midori_location_action_init (MidoriLocationAction* location_action)
 {
     location_action->uri = NULL;
     location_action->progress = 0.0;
+    location_action->secondary_icon = NULL;
     location_action->default_icon = NULL;
 
     location_action->model = (GtkTreeModel*)gtk_list_store_new (N_COLS,
@@ -325,6 +349,28 @@ midori_location_action_set_property (GObject*      object,
     }
 }
 
+static void
+midori_location_action_get_property (GObject*    object,
+                                     guint       prop_id,
+                                     GValue*     value,
+                                     GParamSpec* pspec)
+{
+    MidoriLocationAction* location_action = MIDORI_LOCATION_ACTION (object);
+
+    switch (prop_id)
+    {
+    case PROP_PROGRESS:
+        g_value_set_double (value, location_action->progress);
+        break;
+    case PROP_SECONDARY_ICON:
+        g_value_set_string (value, location_action->secondary_icon);
+        break;
+    default:
+        G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
+        break;
+    }
+}
+
 static void
 midori_location_action_activate (GtkAction* action)
 {
@@ -575,6 +621,20 @@ midori_location_action_set_item (MidoriLocationAction*    location_action,
             FAVICON_COL, new_icon, -1);
 }
 
+static gchar*
+midori_location_action_format_uri (const gchar* uri)
+{
+    gchar* new_uri;
+    size_t len;
+
+    new_uri = g_ascii_strdown (uri, -1);
+    len = strlen (new_uri);
+
+    if (new_uri [len - 1] == '/')
+        new_uri [len - 1] = '\0';
+    return new_uri;
+}
+
 /**
  * midori_location_action_iter_lookup:
  * @location_action: a #MidoriLocationAction
@@ -591,18 +651,48 @@ midori_location_action_iter_lookup (MidoriLocationAction* location_action,
                                     const gchar*          uri,
                                     GtkTreeIter*          iter)
 {
+    GtkTreeModel* model;
     gchar* path;
+    gchar* new_uri;
+    gchar* tmp_uri;
+    gboolean found;
 
-    if ((path = g_hash_table_lookup (location_action->items, uri)))
+    model = location_action->model;
+    found = FALSE;
+
+    new_uri = midori_location_action_format_uri (uri);
+
+    if (midori_location_action_is_frozen (location_action))
     {
-        if (gtk_tree_model_get_iter_from_string (location_action->model,
-                                                 iter, path))
-            return TRUE;
-
-        g_hash_table_remove (location_action->items, uri);
+        if ((path = g_hash_table_lookup (location_action->items, new_uri)))
+        {
+            if (!(found = gtk_tree_model_get_iter_from_string (model, iter, path)))
+            {
+                g_hash_table_remove (location_action->items, new_uri);
+            }
+        }
     }
+    else
+    {
+        if (gtk_tree_model_get_iter_first (model, iter))
+        {
+            tmp_uri = NULL;
+            do
+            {
 
-    return FALSE;
+                gtk_tree_model_get (model, iter, URI_COL, &tmp_uri, -1);
+                found = !g_ascii_strncasecmp (new_uri, tmp_uri, strlen (new_uri));
+                katze_assign (tmp_uri, NULL);
+
+                if (found)
+                    break;
+            }
+            while (gtk_tree_model_iter_next (model, iter));
+        }
+    }
+    g_free (new_uri);
+
+    return found;
 }
 
 /**
@@ -627,12 +717,16 @@ midori_location_action_iter_insert (MidoriLocationAction* location_action,
     {
         GtkTreeModel* model;
         gchar* path;
+        gchar* new_uri;
 
         model = location_action->model;
         gtk_list_store_insert (GTK_LIST_STORE (model), iter, position);
-        path = gtk_tree_model_get_string_from_iter (model,  iter);
-        g_hash_table_insert (location_action->items, g_strdup (uri), path);
-
+        if (midori_location_action_is_frozen (location_action))
+        {
+            new_uri = midori_location_action_format_uri (uri);
+            path = gtk_tree_model_get_string_from_iter (model,  iter);
+            g_hash_table_insert (location_action->items, new_uri, path);
+        }
         return FALSE;
     }
 
@@ -892,13 +986,16 @@ midori_location_action_prepend_item (MidoriLocationAction*    location_action,
                                      MidoriLocationEntryItem* item)
 {
     GtkTreeModel* filter_model;
+    GtkTreeModel* sort_model;
     GtkTreeModel* model;
     GtkTreeIter iter;
+    GtkTreeIter sort_iter;
     GtkTreeIter idx;
     gint n;
     gint visits = 0;
 
     filter_model = location_action->filter_model;
+    sort_model = location_action->sort_model;
     model = location_action->model;
 
     if (midori_location_action_iter_insert (location_action,
@@ -909,18 +1006,25 @@ midori_location_action_prepend_item (MidoriLocationAction*    location_action,
         gtk_list_store_move_before (GTK_LIST_STORE (model), &iter, &idx);
     }
 
-    n = gtk_tree_model_iter_n_children (filter_model, NULL);
-    if (n > MAX_ITEMS)
-    {
-        gtk_tree_model_iter_nth_child (model, &idx, NULL, n - 1);
-        gtk_list_store_set (GTK_LIST_STORE (model),
-                            &idx, VISIBLE_COL, FALSE, -1);
-    }
-
     /* Only increment the visits when we add the uri */
     if (!item->title && !item->favicon)
         gtk_list_store_set (GTK_LIST_STORE (model), &iter, VISITS_COL, ++visits,
                             VISIBLE_COL, TRUE, -1);
+
+    if (filter_model)
+    {
+        n = gtk_tree_model_iter_n_children (filter_model, NULL);
+        if (n > MAX_ITEMS)
+        {
+            gtk_tree_model_iter_nth_child (filter_model, &idx, NULL, n - 1);
+            gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (filter_model),
+                    &sort_iter, &idx);
+            gtk_tree_model_sort_convert_iter_to_child_iter (GTK_TREE_MODEL_SORT (sort_model),
+                    &idx, &sort_iter);
+            gtk_list_store_set (GTK_LIST_STORE (model),
+                    &idx, VISIBLE_COL, FALSE, -1);
+        }
+    }
     midori_location_action_set_item (location_action, &iter, item);
 }
 
@@ -938,19 +1042,16 @@ midori_location_action_append_item (MidoriLocationAction*    location_action,
 {
     GtkTreeModel* model;
     GtkTreeIter iter;
-    gint n;
     gint visits = 0;
 
     model = location_action->model;
 
-    if (!midori_location_action_iter_insert (location_action,
+    if (midori_location_action_iter_insert (location_action,
         item->uri, &iter, G_MAXINT))
-        n = gtk_tree_model_iter_n_children (model, NULL);
-    else
         gtk_tree_model_get (model, &iter, VISITS_COL, &visits, -1);
 
     gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-                        VISITS_COL, ++visits, VISIBLE_COL, n <= MAX_ITEMS, -1);
+                        VISITS_COL, ++visits, VISIBLE_COL, TRUE, -1);
     midori_location_action_set_item (location_action, &iter, item);
 }
 
@@ -1114,6 +1215,8 @@ midori_location_action_set_secondary_icon (MidoriLocationAction* location_action
     g_return_if_fail (MIDORI_IS_LOCATION_ACTION (location_action));
     g_return_if_fail (!stock_id || gtk_stock_lookup (stock_id, &stock_item));
 
+    katze_assign (location_action->secondary_icon, g_strdup (stock_id));
+
     proxies = gtk_action_get_proxies (GTK_ACTION (location_action));
 
     for (; proxies != NULL; proxies = g_slist_next (proxies))
