@@ -28,6 +28,7 @@ struct _MidoriPreferences
 {
     GtkDialog parent_instance;
 
+    MidoriWebSettings* settings;
     GtkWidget* notebook;
 };
 
@@ -70,7 +71,7 @@ midori_preferences_class_init (MidoriPreferencesClass* class)
                                      "Settings",
                                      "Settings instance to provide properties",
                                      MIDORI_TYPE_WEB_SETTINGS,
-                                     G_PARAM_WRITABLE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -86,6 +87,7 @@ midori_preferences_init (MidoriPreferences* preferences)
 {
     gchar* dialog_title;
 
+    preferences->settings = NULL;
     preferences->notebook = NULL;
 
     dialog_title = g_strdup_printf (_("Preferences for %s"),
@@ -123,11 +125,9 @@ midori_preferences_set_property (GObject*      object,
     switch (prop_id)
     {
     case PROP_SETTINGS:
-    {
         midori_preferences_set_settings (preferences,
                                          g_value_get_object (value));
         break;
-    }
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -140,8 +140,13 @@ midori_preferences_get_property (GObject*    object,
                                  GValue*     value,
                                  GParamSpec* pspec)
 {
+    MidoriPreferences* preferences = MIDORI_PREFERENCES (object);
+
     switch (prop_id)
     {
+    case PROP_SETTINGS:
+        g_value_set_object (value, preferences->settings);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
         break;
@@ -224,6 +229,17 @@ midori_preferences_notify_preferred_encoding_cb (MidoriWebSettings* settings,
 }
 
 #if HAVE_LIBSOUP
+static void
+midori_preferences_notify_auto_detect_proxy_cb (MidoriWebSettings* settings,
+                                                GParamSpec*        pspec,
+                                                GtkWidget*         entry)
+{
+    MidoriIdentity auto_detect_proxy = katze_object_get_enum (settings,
+                                                              "auto-detect-proxy");
+
+    gtk_widget_set_sensitive (entry, !auto_detect_proxy);
+}
+
 static void
 midori_preferences_notify_identify_as_cb (MidoriWebSettings* settings,
                                           GParamSpec*        pspec,
@@ -300,12 +316,17 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     GtkWidget* entry;
     GtkWidget* hbox;
     gint icon_width, icon_height;
+    #if HAVE_LIBSOUP
+    GObjectClass* webkit_class;
+    #endif
 
     g_return_if_fail (MIDORI_IS_PREFERENCES (preferences));
     g_return_if_fail (MIDORI_IS_WEB_SETTINGS (settings));
 
     g_return_if_fail (!preferences->notebook);
 
+    katze_assign (preferences->settings, g_object_ref (settings));
+
     g_object_get (preferences, "transient-for", &parent, NULL);
     icon_name = parent ? gtk_window_get_icon_name (parent) : NULL;
     if ((header = sokoke_xfce_header_new (icon_name,
@@ -450,7 +471,7 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     /* Page "Behavior" */
     PAGE_NEW (GTK_STOCK_SELECT_COLOR, _("Behavior"));
     FRAME_NEW (_("Features"));
-    TABLE_NEW (6, 2);
+    TABLE_NEW (7, 2);
     button = katze_property_proxy (settings, "auto-load-images", NULL);
     gtk_button_set_label (GTK_BUTTON (button), _("Load images automatically"));
     gtk_widget_set_tooltip_text (button, _("Load and display images automatically"));
@@ -483,47 +504,46 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
     gtk_button_set_label (GTK_BUTTON (button), _("Enable developer tools"));
     gtk_widget_set_tooltip_text (button, _("Enable special extensions for developers"));
     SPANNED_ADD (button, 1, 2, 3, 4);
+    button = katze_property_proxy (settings, "zoom-text-and-images", NULL);
+    SPANNED_ADD (button, 0, 1, 4, 5);
     label = katze_property_label (settings, "location-entry-search");
-    INDENTED_ADD (label, 0, 1, 4, 5);
+    INDENTED_ADD (label, 0, 1, 5, 6);
     entry = katze_property_proxy (settings, "location-entry-search", NULL);
-    FILLED_ADD (entry, 1, 2, 4, 5);
+    FILLED_ADD (entry, 1, 2, 5, 6);
 
     /* Page "Interface" */
     PAGE_NEW (GTK_STOCK_CONVERT, _("Interface"));
     FRAME_NEW (_("Navigationbar"));
-    TABLE_NEW (3, 2);
+    TABLE_NEW (2, 2);
+    #if !HAVE_HILDON
     INDENTED_ADD (katze_property_label (settings, "toolbar-style"), 0, 1, 0, 1);
     button = katze_property_proxy (settings, "toolbar-style", NULL);
     FILLED_ADD (button, 1, 2, 0, 1);
-    INDENTED_ADD (katze_property_label (settings, "toolbar-items"), 0, 1, 1, 2);
-    button = katze_property_proxy (settings, "toolbar-items", NULL);
+    #endif
+    button = katze_property_proxy (settings, "progress-in-location", NULL);
     FILLED_ADD (button, 1, 2, 1, 2);
-    button = gtk_label_new (NULL);
-    gtk_label_set_markup (GTK_LABEL (button),
-        "<span size='smaller'>"
-        "Separator, TabNew, Open, Print, Find, Location, Preferences, Window, Bookmarks\n"
-        "ReloadStop, ZoomIn, ZoomOut, Back, Forward, Homepage, Search, Trash"
-        "</span>");
-    gtk_label_set_selectable (GTK_LABEL (button), TRUE);
-    FILLED_ADD (button, 0, 2, 2, 3);
     FRAME_NEW (_("Browsing"));
     TABLE_NEW (5, 2);
     label = katze_property_label (settings, "open-new-pages-in");
     INDENTED_ADD (label, 0, 1, 0, 1);
     button = katze_property_proxy (settings, "open-new-pages-in", NULL);
     FILLED_ADD (button, 1, 2, 0, 1);
+    #if HAVE_UNIQUE
     label = katze_property_label (settings, "open-external-pages-in");
     INDENTED_ADD (label, 0, 1, 1, 2);
     button = katze_property_proxy (settings, "open-external-pages-in", NULL);
     FILLED_ADD (button, 1, 2, 1, 2);
+    #endif
     button = katze_property_proxy (settings, "always-show-tabbar", NULL);
     INDENTED_ADD (button, 0, 1, 2, 3);
-    button = katze_property_proxy (settings, "compact-sidepanel", NULL);
+    button = katze_property_proxy (settings, "open-tabs-in-the-background", NULL);
     INDENTED_ADD (button, 1, 2, 2, 3);
+    #if !HAVE_HILDON
     button = katze_property_proxy (settings, "middle-click-opens-selection", NULL);
     INDENTED_ADD (button, 0, 1, 3, 4);
-    button = katze_property_proxy (settings, "open-tabs-in-the-background", NULL);
+    button = katze_property_proxy (settings, "compact-sidepanel", NULL);
     WIDGET_ADD (button, 1, 2, 3, 4);
+    #endif
     /* button = katze_property_proxy (settings, "open-popups-in-tabs", NULL);
     SPANNED_ADD (button, 0, 1, 4, 5);*/
     button = katze_property_proxy (settings, "open-tabs-next-to-current", NULL);
@@ -533,44 +553,52 @@ midori_preferences_set_settings (MidoriPreferences* preferences,
 
     /* Page "Network" */
     #if HAVE_LIBSOUP
+    webkit_class = g_type_class_ref (WEBKIT_TYPE_WEB_VIEW);
+    if (g_object_class_find_property (webkit_class, "session") ||
     /* If a cookie jar was created, WebKit is using Soup */
-    if (g_type_get_qdata (SOUP_TYPE_COOKIE_JAR,
+        g_type_get_qdata (SOUP_TYPE_COOKIE_JAR,
         g_quark_from_static_string ("midori-has-jar")))
     {
     PAGE_NEW (GTK_STOCK_NETWORK, _("Network"));
     FRAME_NEW (_("Network"));
-    TABLE_NEW (4, 2);
+    TABLE_NEW (5, 2);
     label = katze_property_label (settings, "http-proxy");
     INDENTED_ADD (label, 0, 1, 0, 1);
-    button = katze_property_proxy (settings, "http-proxy", NULL);
-    FILLED_ADD (button, 1, 2, 0, 1);
-    label = katze_property_label (settings, "identify-as");
-    INDENTED_ADD (label, 0, 1, 1, 2);
-    button = katze_property_proxy (settings, "identify-as", NULL);
+    entry = katze_property_proxy (settings, "http-proxy", NULL);
+    FILLED_ADD (entry, 1, 2, 0, 1);
+    button = katze_property_proxy (settings, "auto-detect-proxy", NULL);
     FILLED_ADD (button, 1, 2, 1, 2);
-    label = katze_property_label (settings, "ident-string");
+    g_signal_connect (settings, "notify::auto-detect-proxy",
+        G_CALLBACK (midori_preferences_notify_auto_detect_proxy_cb), entry);
+    midori_preferences_notify_auto_detect_proxy_cb (settings, NULL, entry);
+    label = katze_property_label (settings, "identify-as");
     INDENTED_ADD (label, 0, 1, 2, 3);
+    button = katze_property_proxy (settings, "identify-as", NULL);
+    FILLED_ADD (button, 1, 2, 2, 3);
+    label = katze_property_label (settings, "ident-string");
+    INDENTED_ADD (label, 0, 1, 3, 4);
     entry = katze_property_proxy (settings, "ident-string", NULL);
     g_signal_connect (settings, "notify::identify-as",
         G_CALLBACK (midori_preferences_notify_identify_as_cb), entry);
     midori_preferences_notify_identify_as_cb (settings, NULL, entry);
-    FILLED_ADD (entry, 1, 2, 2, 3);
+    FILLED_ADD (entry, 1, 2, 3, 4);
     label = katze_property_label (settings, "cache-size");
-    INDENTED_ADD (label, 0, 1, 3, 4);
+    INDENTED_ADD (label, 0, 1, 4, 5);
     hbox = gtk_hbox_new (FALSE, 4);
     entry = katze_property_proxy (settings, "cache-size", NULL);
     gtk_box_pack_start (GTK_BOX (hbox), entry, FALSE, FALSE, 0);
     gtk_box_pack_start (GTK_BOX (hbox), gtk_label_new (_("MB")),
                         FALSE, FALSE, 0);
-    FILLED_ADD (hbox, 1, 2, 3, 4);
+    FILLED_ADD (hbox, 1, 2, 4, 5);
     }
     #endif
 
     /* Page "Privacy" */
     PAGE_NEW (GTK_STOCK_INDEX, _("Privacy"));
     #if HAVE_LIBSOUP_2_25_2
+    if (g_object_class_find_property (webkit_class, "session") ||
     /* If a cookie jar was created, WebKit is using Soup */
-    if (g_type_get_qdata (SOUP_TYPE_COOKIE_JAR,
+        g_type_get_qdata (SOUP_TYPE_COOKIE_JAR,
         g_quark_from_static_string ("midori-has-jar")))
     {
     FRAME_NEW (_("Web Cookies"));
