@@ -136,7 +136,7 @@ midori_search_action_class_init (MidoriSearchActionClass* class)
                                      "Search Engines",
                                      "The list of search engines",
                                      KATZE_TYPE_ARRAY,
-                                     G_PARAM_READWRITE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_CURRENT_ITEM,
@@ -145,7 +145,7 @@ midori_search_action_class_init (MidoriSearchActionClass* class)
                                      "Current Item",
                                      "The currently selected item",
                                      KATZE_TYPE_ITEM,
-                                     G_PARAM_READWRITE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_TEXT,
@@ -154,7 +154,7 @@ midori_search_action_class_init (MidoriSearchActionClass* class)
                                      "Text",
                                      "The current text typed in the entry",
                                      NULL,
-                                     G_PARAM_READWRITE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_DIALOG,
@@ -163,7 +163,7 @@ midori_search_action_class_init (MidoriSearchActionClass* class)
                                      "Dialog",
                                      "A dialog to manage search engines",
                                      GTK_TYPE_DIALOG,
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -389,6 +389,7 @@ midori_search_action_get_icon (MidoriSearchAction* search_action,
         GdkScreen* screen;
         GtkIconTheme* icon_theme;
         gint width, height;
+        GdkPixbuf* pixbuf;
 
         if (G_UNLIKELY (!(screen = gtk_widget_get_screen (widget))))
             return gtk_widget_render_icon (widget, GTK_STOCK_FILE,
@@ -396,8 +397,9 @@ midori_search_action_get_icon (MidoriSearchAction* search_action,
         icon_theme = gtk_icon_theme_get_for_screen (screen);
         gtk_icon_size_lookup_for_settings (gtk_widget_get_settings (widget),
             GTK_ICON_SIZE_MENU, &width, &height);
-        return gtk_icon_theme_load_icon (icon_theme, icon, MAX (width, height),
-                                         GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
+        if ((pixbuf = gtk_icon_theme_load_icon (icon_theme, icon, MAX (width, height),
+                                           GTK_ICON_LOOKUP_USE_BUILTIN, NULL)))
+            return pixbuf;
     }
 
     if ((icon = katze_item_get_uri (item)) && strstr (icon, "://"))
@@ -577,7 +579,6 @@ midori_search_action_set_text (MidoriSearchAction* search_action,
     GtkWidget* entry;
 
     g_return_if_fail (MIDORI_IS_SEARCH_ACTION (search_action));
-    g_return_if_fail (text != NULL);
 
     katze_assign (search_action->text, g_strdup (text));
     g_object_notify (G_OBJECT (search_action), "text");
@@ -592,7 +593,7 @@ midori_search_action_set_text (MidoriSearchAction* search_action,
         alignment = gtk_bin_get_child (GTK_BIN (proxies->data));
         entry = gtk_bin_get_child (GTK_BIN (alignment));
 
-        gtk_entry_set_text (GTK_ENTRY (entry), text);
+        gtk_entry_set_text (GTK_ENTRY (entry), text ? text : "");
         search_action->last_proxy = proxies->data;
     }
     while ((proxies = g_slist_next (proxies)));
@@ -647,6 +648,8 @@ midori_search_action_set_search_engines (MidoriSearchAction* search_action,
     if (search_engines)
         g_object_ref (search_engines);
     katze_object_assign (search_action->search_engines, search_engines);
+    if (!search_engines)
+        return;
 
     g_object_connect (search_engines,
         "signal-after::add-item",
@@ -727,7 +730,7 @@ midori_search_action_dialog_render_icon_cb (GtkTreeViewColumn* column,
 
     search_action = g_object_get_data (G_OBJECT (treeview), "search-action");
     icon = midori_search_action_get_icon (search_action, item, treeview);
-    g_object_set (renderer, "pixbuf", icon, NULL);
+    g_object_set (renderer, "pixbuf", icon, "yalign", 0.25, NULL);
     g_object_unref (icon);
 }
 
@@ -751,6 +754,24 @@ midori_search_action_dialog_render_text (GtkTreeViewColumn* column,
     g_free (markup);
 }
 
+static void
+midori_search_action_dialog_render_token (GtkTreeViewColumn* column,
+                                          GtkCellRenderer*   renderer,
+                                          GtkTreeModel*      model,
+                                          GtkTreeIter*       iter,
+                                          GtkWidget*         treeview)
+{
+    KatzeItem* item;
+    const gchar* token;
+    gchar* markup;
+
+    gtk_tree_model_get (model, iter, 0, &item, -1);
+    token = katze_item_get_token (item);
+    markup = g_markup_printf_escaped ("<b>%s</b>", token ? token : "");
+    g_object_set (renderer, "markup", markup, "yalign", 0.0, NULL);
+    g_free (markup);
+}
+
 static void
 midori_search_action_editor_name_changed_cb (GtkWidget* entry,
                                              GtkWidget* dialog)
@@ -859,7 +880,7 @@ midori_search_action_get_editor (MidoriSearchAction* search_action,
 
     hbox = gtk_hbox_new (FALSE, 8);
     gtk_container_set_border_width (GTK_CONTAINER (hbox), 5);
-    label = gtk_label_new_with_mnemonic (_("_Icon (name or file):"));
+    label = gtk_label_new_with_mnemonic (_("_Icon:"));
     gtk_size_group_add_widget (sizegroup, label);
     gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
     entry_icon = gtk_entry_new ();
@@ -1051,7 +1072,8 @@ midori_search_action_get_dialog (MidoriSearchAction* search_action)
         return search_action->dialog;
 
     dialog_title = _("Manage Search Engines");
-    toplevel = gtk_widget_get_toplevel (search_action->last_proxy);
+    toplevel = search_action->last_proxy ?
+        gtk_widget_get_toplevel (search_action->last_proxy) : NULL;
     dialog = gtk_dialog_new_with_buttons (dialog_title,
         toplevel ? GTK_WINDOW (toplevel) : NULL,
         GTK_DIALOG_DESTROY_WITH_PARENT | GTK_DIALOG_NO_SEPARATOR,
@@ -1098,6 +1120,11 @@ midori_search_action_get_dialog (MidoriSearchAction* search_action)
     gtk_tree_view_column_set_cell_data_func (column, renderer_text,
         (GtkTreeCellDataFunc)midori_search_action_dialog_render_text,
         treeview, NULL);
+    renderer_text = gtk_cell_renderer_text_new ();
+    gtk_tree_view_column_pack_start (column, renderer_text, TRUE);
+    gtk_tree_view_column_set_cell_data_func (column, renderer_text,
+        (GtkTreeCellDataFunc)midori_search_action_dialog_render_token,
+        treeview, NULL);
     gtk_tree_view_append_column (GTK_TREE_VIEW (treeview), column);
     scrolled = gtk_scrolled_window_new (NULL, NULL);
     gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
@@ -1106,7 +1133,8 @@ midori_search_action_get_dialog (MidoriSearchAction* search_action)
     gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled),
                                          GTK_SHADOW_IN);
     gtk_box_pack_start (GTK_BOX (hbox), scrolled, TRUE, TRUE, 5);
-    n = katze_array_get_length (search_action->search_engines);
+    n = search_action->search_engines ?
+        katze_array_get_length (search_action->search_engines) : 0;
     for (i = 0; i < n; i++)
     {
         item = katze_array_get_nth_item (search_action->search_engines, i);
@@ -1162,12 +1190,13 @@ midori_search_action_get_dialog (MidoriSearchAction* search_action)
     #endif
     gtk_widget_show_all (GTK_DIALOG (dialog)->vbox);
 
-    g_object_connect (search_action->search_engines,
-        "signal-after::add-item",
-        midori_search_action_dialog_engines_add_item_cb, search_action,
-        "signal-after::remove-item",
-        midori_search_action_dialog_engines_remove_item_cb, search_action,
-        NULL);
+    if (search_action->search_engines)
+        g_object_connect (search_action->search_engines,
+            "signal-after::add-item",
+            midori_search_action_dialog_engines_add_item_cb, search_action,
+            "signal-after::remove-item",
+            midori_search_action_dialog_engines_remove_item_cb, search_action,
+            NULL);
 
     search_action->dialog = dialog;
     return dialog;
