@@ -26,6 +26,14 @@
 #include <glib/gi18n.h>
 #include <webkit/webkit.h>
 
+#if GLIB_CHECK_VERSION (2, 18, 0)
+    #define D_(__domain, __message) g_dgettext (__domain, __message)
+#elif ENABLE_NLS
+    #define D_(__domain, __message) dgettext (__domain, __message)
+#else
+    #define D_(__domain, __message) __message
+#endif
+
 #ifndef WEBKIT_CHECK_VERSION
     #define WEBKIT_CHECK_VERSION(a,b,c) 0
 #endif
@@ -267,6 +275,18 @@ midori_view_class_init (MidoriViewClass* class)
         MIDORI_TYPE_VIEW,
         MIDORI_TYPE_NEW_VIEW);
 
+    /**
+     * MidoriView::search-text:
+     * @view: the object on which the signal is emitted
+     * @found: whether the search was successful
+     * @typing: whether the search was initiated by typing
+     *
+     * Emitted when a search is performed. Either manually
+     * invoked or automatically by typing. The value of typing
+     * is actually the text the user typed.
+     *
+     * Note that in 0.1.3 the argument @typing was introduced.
+     */
     signals[SEARCH_TEXT] = g_signal_new (
         "search-text",
         G_TYPE_FROM_CLASS (class),
@@ -274,9 +294,10 @@ midori_view_class_init (MidoriViewClass* class)
         0,
         0,
         NULL,
-        g_cclosure_marshal_VOID__BOOLEAN,
-        G_TYPE_NONE, 1,
-        G_TYPE_BOOLEAN);
+        midori_cclosure_marshal_VOID__BOOLEAN_STRING,
+        G_TYPE_NONE, 2,
+        G_TYPE_BOOLEAN,
+        G_TYPE_STRING);
 
     signals[ADD_BOOKMARK] = g_signal_new (
         "add-bookmark",
@@ -305,7 +326,7 @@ midori_view_class_init (MidoriViewClass* class)
     gobject_class->set_property = midori_view_set_property;
     gobject_class->get_property = midori_view_get_property;
 
-    flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT;
+    flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS;
 
     g_object_class_install_property (gobject_class,
                                      PROP_URI,
@@ -314,7 +335,7 @@ midori_view_class_init (MidoriViewClass* class)
                                      "Uri",
                                      "The URI of the currently loaded page",
                                      "about:blank",
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_TITLE,
@@ -323,7 +344,7 @@ midori_view_class_init (MidoriViewClass* class)
                                      "Title",
                                      "The title of the currently loaded page",
                                      NULL,
-                                     G_PARAM_READWRITE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
     /**
     * MidoriView:mime-type:
@@ -339,7 +360,7 @@ midori_view_class_init (MidoriViewClass* class)
                                      "MIME Type",
                                      "The MIME type of the currently loaded page",
                                      "text/html",
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_ICON,
@@ -348,7 +369,7 @@ midori_view_class_init (MidoriViewClass* class)
                                      "Icon",
                                      "The icon of the view",
                                      GDK_TYPE_PIXBUF,
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_LOAD_STATUS,
@@ -358,7 +379,7 @@ midori_view_class_init (MidoriViewClass* class)
                                      "The current loading status",
                                      MIDORI_TYPE_LOAD_STATUS,
                                      MIDORI_LOAD_FINISHED,
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_PROGRESS,
@@ -367,7 +388,7 @@ midori_view_class_init (MidoriViewClass* class)
                                      "Progress",
                                      "The current loading progress",
                                      0.0, 1.0, 0.0,
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_ZOOM_LEVEL,
@@ -378,7 +399,7 @@ midori_view_class_init (MidoriViewClass* class)
                                      G_MINFLOAT,
                                      G_MAXFLOAT,
                                      1.0f,
-                                     G_PARAM_READWRITE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
     /* g_object_class_install_property (gobject_class,
                                      PROP_NEWS_FEEDS,
@@ -387,7 +408,7 @@ midori_view_class_init (MidoriViewClass* class)
                                      "News Feeds",
                                      "The list of available news feeds",
                                      KATZE_TYPE_ARRAY,
-                                     G_PARAM_READWRITE)); */
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS)); */
 
     g_object_class_install_property (gobject_class,
                                      PROP_STATUSBAR_TEXT,
@@ -396,7 +417,7 @@ midori_view_class_init (MidoriViewClass* class)
                                      "Statusbar Text",
                                      "The text displayed in the statusbar",
                                      "",
-                                     G_PARAM_READWRITE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_SETTINGS,
@@ -543,7 +564,13 @@ webkit_web_view_load_committed_cb (WebKitWebView*  web_view,
     g_object_freeze_notify (G_OBJECT (view));
 
     uri = webkit_web_frame_get_uri (web_frame);
+    g_return_if_fail (uri != NULL);
     katze_assign (view->uri, g_strdup (uri));
+    if (view->item)
+    {
+        katze_item_set_uri (view->item, uri);
+        katze_item_set_added (view->item, time (NULL));
+    }
     g_object_notify (G_OBJECT (view), "uri");
     g_object_set (view, "title", NULL, NULL);
 
@@ -780,6 +807,28 @@ gtk_widget_button_press_event_cb (WebKitWebView*  web_view,
     return FALSE;
 }
 
+static gboolean
+gtk_widget_key_press_event_cb (WebKitWebView* web_view,
+                               GdkEventKey*   event,
+                               MidoriView*    view)
+{
+    guint character = gdk_unicode_to_keyval (event->keyval);
+    /* Skip control characters */
+    if (character == (event->keyval | 0x01000000))
+        return FALSE;
+
+    if (!webkit_web_view_can_cut_clipboard (web_view)
+        && !webkit_web_view_can_paste_clipboard (web_view))
+    {
+        gchar* text = g_strdup_printf ("%c", character);
+
+        g_signal_emit (view, signals[SEARCH_TEXT], 0, TRUE, text);
+        g_free (text);
+    }
+
+    return FALSE;
+}
+
 static gboolean
 gtk_widget_scroll_event_cb (WebKitWebView*  web_view,
                             GdkEventScroll* event,
@@ -818,6 +867,29 @@ midori_web_view_menu_new_window_activate_cb (GtkWidget*  widget,
     g_signal_emit (view, signals[NEW_WINDOW], 0, view->link_uri);
 }
 
+static void
+midori_web_view_menu_search_web_activate_cb (GtkWidget*  widget,
+                                             MidoriView* view)
+{
+    gchar* uri;
+    gchar* location_entry_search;
+
+    g_object_get (view->settings, "location-entry-search",
+                  &location_entry_search, NULL);
+    if (strstr (location_entry_search, "%s"))
+    {
+        uri = g_strdup_printf (location_entry_search, view->selected_text);
+        g_free (location_entry_search);
+    }
+    else
+        uri = location_entry_search;
+
+    g_signal_emit (view, signals[NEW_TAB], 0, uri,
+        view->open_tabs_in_the_background);
+
+    g_free (uri);
+}
+
 static void
 midori_web_view_menu_save_as_activate_cb (GtkWidget*  widget,
                                           MidoriView* view)
@@ -859,6 +931,7 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
     GtkIconTheme* icon_theme;
     GList* items;
     gboolean has_selection;
+    GtkWidget* label;
 
     has_selection = midori_view_has_selection (view);
 
@@ -866,6 +939,7 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
        determine that the mouse is over a text area or selection. */
     items = gtk_container_get_children (GTK_CONTAINER (menu));
     menuitem = (GtkWidget*)g_list_nth_data (items, 0);
+    g_list_free (items);
     if (GTK_IS_IMAGE_MENU_ITEM (menuitem))
     {
         icon = gtk_image_menu_item_get_image (GTK_IMAGE_MENU_ITEM (menuitem));
@@ -878,6 +952,11 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
 
     if (view->link_uri)
     {
+        items = gtk_container_get_children (GTK_CONTAINER (menu));
+        menuitem = (GtkWidget*)g_list_nth_data (items, 0);
+        /* hack to localize menu item */
+        label = gtk_bin_get_child (GTK_BIN (menuitem));
+        gtk_label_set_label (GTK_LABEL (label), _("Open _Link"));
         menuitem = gtk_image_menu_item_new_with_mnemonic (
             _("Open Link in New _Tab"));
         screen = gtk_widget_get_screen (GTK_WIDGET (view));
@@ -891,9 +970,13 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
         g_signal_connect (menuitem, "activate",
             G_CALLBACK (midori_web_view_menu_new_tab_activate_cb), view);
         gtk_widget_show (menuitem);
-        /* hack to implement New Window */
+        g_list_free (items);
         items = gtk_container_get_children (GTK_CONTAINER (menu));
         menuitem = (GtkWidget*)g_list_nth_data (items, 2);
+        /* hack to localize menu item */
+        label = gtk_bin_get_child (GTK_BIN (menuitem));
+        gtk_label_set_label (GTK_LABEL (label), _("Open Link in New _Window"));
+        /* hack to implement New Window */
         g_signal_connect (menuitem, "activate",
             G_CALLBACK (midori_web_view_menu_new_window_activate_cb), view);
         menuitem = (GtkWidget*)g_list_nth_data (items, 3);
@@ -928,6 +1011,15 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
 
     if (!view->link_uri && has_selection)
     {
+        items = gtk_container_get_children (GTK_CONTAINER (menu));
+        menuitem = (GtkWidget*)g_list_nth_data (items, 0);
+        /* hack to localize menu item */
+        label = gtk_bin_get_child (GTK_BIN (menuitem));
+        gtk_label_set_label (GTK_LABEL (label), _("_Search the Web"));
+        /* hack to implement Search the Web */
+        g_signal_connect (menuitem, "activate",
+            G_CALLBACK (midori_web_view_menu_search_web_activate_cb), view);
+        g_list_free (items);
         if (strchr (view->selected_text, '.')
             && !strchr (view->selected_text, ' '))
         {
@@ -947,6 +1039,12 @@ webkit_web_view_populate_popup_cb (WebKitWebView* web_view,
 
     if (!view->link_uri && !has_selection)
     {
+        items = gtk_container_get_children (GTK_CONTAINER (menu));
+        menuitem = (GtkWidget*)g_list_nth_data (items, 3);
+        /* hack to localize menu item */
+        label = gtk_bin_get_child (GTK_BIN (menuitem));
+        gtk_label_set_label (GTK_LABEL (label), D_("gtk20", "_Refresh"));
+        g_list_free (items);
         menuitem = gtk_image_menu_item_new_with_mnemonic (_("Undo Close Tab"));
         icon = gtk_image_new_from_stock (GTK_STOCK_UNDELETE, GTK_ICON_SIZE_MENU);
         gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menuitem), icon);
@@ -1096,14 +1194,6 @@ webkit_web_view_window_object_cleared_cb (GtkWidget*      web_view,
     g_signal_emit (view, signals[CONTEXT_READY], 0, js_context);
 }
 
-static void
-webkit_web_view_destroy_cb (GtkWidget*      web_view,
-                            WebKitWebFrame* web_frame)
-{
-    g_signal_handlers_disconnect_by_func (web_frame,
-        webkit_web_frame_load_done_cb, gtk_widget_get_parent (web_view));
-}
-
 static void
 midori_view_init (MidoriView* view)
 {
@@ -1217,6 +1307,12 @@ midori_view_get_property (GObject*    object,
     case PROP_TITLE:
         g_value_set_string (value, view->title);
         break;
+    case PROP_MIME_TYPE:
+        g_value_set_string (value, view->mime_type);
+        break;
+    case PROP_ICON:
+        g_value_set_object (value, view->icon);
+        break;
     case PROP_PROGRESS:
         g_value_set_double (value, midori_view_get_progress (view));
         break;
@@ -1260,13 +1356,20 @@ midori_view_new (KatzeNet* net)
 static void
 _midori_view_update_settings (MidoriView* view)
 {
+    gboolean zoom_text_and_images;
+
     g_object_get (view->settings,
         "download-manager", &view->download_manager,
+        "zoom-text-and-images", &zoom_text_and_images,
         "close-buttons-on-tabs", &view->close_buttons_on_tabs,
         "open-new-pages-in", &view->open_new_pages_in,
         "middle-click-opens-selection", &view->middle_click_opens_selection,
         "open-tabs-in-the-background", &view->open_tabs_in_the_background,
         NULL);
+
+    if (view->web_view)
+        g_object_set (view->web_view, "full-content-zoom",
+                      zoom_text_and_images, NULL);
 }
 
 static void
@@ -1285,6 +1388,12 @@ midori_view_settings_notify_cb (MidoriWebSettings* settings,
     {
         katze_assign (view->download_manager, g_value_dup_string (&value));
     }
+    else if (name == g_intern_string ("zoom-text-and-images"))
+    {
+        if (view->web_view)
+            g_object_set (view->web_view, "full-content-zoom",
+                          g_value_get_boolean (&value), NULL);
+    }
     else if (name == g_intern_string ("close-buttons-on-tabs"))
     {
         view->close_buttons_on_tabs = g_value_get_boolean (&value);
@@ -1442,6 +1551,11 @@ midori_view_construct_web_view (MidoriView* view)
     gpointer inspector;
 
     view->web_view = webkit_web_view_new ();
+    if (g_object_class_find_property (G_OBJECT_GET_CLASS (view->web_view), "session"))
+    #if HAVE_LIBSOUP
+        g_object_set (view->web_view, "session",
+                      katze_net_get_session (view->net), NULL);
+    #endif
 
     web_frame = webkit_web_view_get_main_frame (WEBKIT_WEB_VIEW (view->web_view));
 
@@ -1462,6 +1576,8 @@ midori_view_construct_web_view (MidoriView* view)
                       webkit_web_view_hovering_over_link_cb, view,
                       "signal::button-press-event",
                       gtk_widget_button_press_event_cb, view,
+                      "signal::key-press-event",
+                      gtk_widget_key_press_event_cb, view,
                       "signal::scroll-event",
                       gtk_widget_scroll_event_cb, view,
                       "signal::populate-popup",
@@ -1470,8 +1586,6 @@ midori_view_construct_web_view (MidoriView* view)
                       webkit_web_view_console_message_cb, view,
                       "signal::window-object-cleared",
                       webkit_web_view_window_object_cleared_cb, view,
-                      "signal::destroy",
-                      webkit_web_view_destroy_cb, web_frame,
                       NULL);
     if (g_signal_lookup ("create-web-view", WEBKIT_TYPE_WEB_VIEW))
         g_object_connect (view->web_view,
@@ -1489,6 +1603,9 @@ midori_view_construct_web_view (MidoriView* view)
                       NULL);
 
     g_object_set (view->web_view, "settings", view->settings, NULL);
+    if (katze_object_has_property (view->web_view, "full-content-zoom"))
+        g_object_set (view->web_view, "full-content-zoom",
+        katze_object_get_boolean (view->settings, "zoom-text-and-images"), NULL);
 
     gtk_widget_show (view->web_view);
     gtk_container_add (GTK_CONTAINER (view), view->web_view);
@@ -1532,6 +1649,8 @@ midori_view_set_uri (MidoriView*  view,
     {
         katze_assign (view->uri, g_strdup (uri));
         g_object_notify (G_OBJECT (view), "uri");
+        if (view->item)
+            katze_item_set_uri (view->item, uri);
         data = g_strdup_printf ("%s - %s", _("Source"), &uri[12]);
         g_object_set (view, "title", data, NULL);
         g_free (data);
@@ -1592,6 +1711,8 @@ midori_view_set_uri (MidoriView*  view,
                     WEBKIT_WEB_VIEW (view->web_view), data, view->uri);
                 g_free (data);
                 g_object_notify (G_OBJECT (view), "uri");
+                if (view->item)
+                    katze_item_set_uri (view->item, uri);
                 return;
             }
         }
@@ -1602,6 +1723,9 @@ midori_view_set_uri (MidoriView*  view,
         else
         {
             katze_assign (view->uri, g_strdup (uri));
+            g_object_notify (G_OBJECT (view), "uri");
+            if (view->item)
+                katze_item_set_uri (view->item, uri);
             webkit_web_view_open (WEBKIT_WEB_VIEW (view->web_view), uri);
         }
     }
@@ -1725,8 +1849,8 @@ midori_view_has_selection (MidoriView* view)
 {
     g_return_val_if_fail (MIDORI_IS_VIEW (view), FALSE);
 
-    view->selected_text = webkit_web_view_get_selected_text (
-            WEBKIT_WEB_VIEW (view->web_view));
+    katze_assign (view->selected_text, webkit_web_view_get_selected_text (
+        WEBKIT_WEB_VIEW (view->web_view)));
     if (view->selected_text && *view->selected_text)
         return TRUE;
     else
@@ -1747,9 +1871,8 @@ midori_view_get_selected_text (MidoriView* view)
     g_return_val_if_fail (MIDORI_IS_VIEW (view), NULL);
 
     if (midori_view_has_selection (view))
-        return midori_view_get_selected_text (view);
-    else
-        return NULL;
+        return view->selected_text;
+    return NULL;
 }
 
 /**
@@ -2348,7 +2471,7 @@ midori_view_search_text (MidoriView*  view,
 
     g_signal_emit (view, signals[SEARCH_TEXT], 0,
         webkit_web_view_search_text (WEBKIT_WEB_VIEW (view->web_view),
-            text, case_sensitive, forward, TRUE));
+            text, case_sensitive, forward, TRUE), NULL);
 }
 
 /**
