@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2008 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2008-2009 Christian Dywan <christian@twotoasts.de>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -27,6 +27,7 @@ struct _MidoriWebSettings
     gboolean remember_last_window_size;
     gint last_window_width;
     gint last_window_height;
+    MidoriWindowState last_window_state;
     gint last_panel_position;
     gint last_panel_page;
     gint last_web_search;
@@ -38,8 +39,10 @@ struct _MidoriWebSettings
     gboolean show_statusbar;
 
     MidoriToolbarStyle toolbar_style;
+    gboolean progress_in_location;
     gchar* toolbar_items;
     gboolean compact_sidepanel;
+    gboolean right_align_sidepanel;
 
     MidoriStartup load_on_startup;
     gchar* homepage;
@@ -59,6 +62,7 @@ struct _MidoriWebSettings
     gboolean open_tabs_next_to_current;
     gboolean open_popups_in_tabs;
 
+    gboolean zoom_text_and_images;
     MidoriAcceptCookies accept_cookies;
     gboolean original_cookies_only;
     gint maximum_cookie_age;
@@ -69,6 +73,7 @@ struct _MidoriWebSettings
     gboolean remember_last_downloaded_files;
 
     gchar* http_proxy;
+    gboolean auto_detect_proxy;
     MidoriIdentity identify_as;
     gchar* ident_string;
     gint cache_size;
@@ -88,6 +93,7 @@ enum
     PROP_REMEMBER_LAST_WINDOW_SIZE,
     PROP_LAST_WINDOW_WIDTH,
     PROP_LAST_WINDOW_HEIGHT,
+    PROP_LAST_WINDOW_STATE,
     PROP_LAST_PANEL_POSITION,
     PROP_LAST_PANEL_PAGE,
     PROP_LAST_WEB_SEARCH,
@@ -99,8 +105,10 @@ enum
     PROP_SHOW_STATUSBAR,
 
     PROP_TOOLBAR_STYLE,
+    PROP_PROGRESS_IN_LOCATION,
     PROP_TOOLBAR_ITEMS,
     PROP_COMPACT_SIDEPANEL,
+    PROP_RIGHT_ALIGN_SIDEPANEL,
 
     PROP_LOAD_ON_STARTUP,
     PROP_HOMEPAGE,
@@ -122,6 +130,7 @@ enum
 
     PROP_ENFORCE_96_DPI,
     PROP_ENABLE_DEVELOPER_EXTRAS,
+    PROP_ZOOM_TEXT_AND_IMAGES,
     PROP_ACCEPT_COOKIES,
     PROP_ORIGINAL_COOKIES_ONLY,
     PROP_MAXIMUM_COOKIE_AGE,
@@ -132,11 +141,30 @@ enum
     PROP_REMEMBER_LAST_DOWNLOADED_FILES,
 
     PROP_HTTP_PROXY,
+    PROP_AUTO_DETECT_PROXY,
     PROP_IDENTIFY_AS,
     PROP_IDENT_STRING,
     PROP_CACHE_SIZE
 };
 
+GType
+midori_window_state_get_type (void)
+{
+    static GType type = 0;
+    if (!type)
+    {
+        static const GEnumValue values[] = {
+         { MIDORI_WINDOW_NORMAL, "MIDORI_WINDOW_NORMAL", "Normal" },
+         { MIDORI_WINDOW_MINIMIZED, "MIDORI_WINDOW_MINIMIZED", "Minimized" },
+         { MIDORI_WINDOW_MAXIMIZED, "MIDORI_WINDOW_MAXIMIZED", "Maximized" },
+         { MIDORI_WINDOW_FULLSCREEN, "MIDORI_WINDOW_FULLSCREEN", "Fullscreen" },
+         { 0, NULL, NULL }
+        };
+        type = g_enum_register_static ("MidoriWindowState", values);
+    }
+    return type;
+}
+
 GType
 midori_startup_get_type (void)
 {
@@ -272,7 +300,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
     gobject_class->set_property = midori_web_settings_set_property;
     gobject_class->get_property = midori_web_settings_get_property;
 
-    flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT;
+    flags = G_PARAM_READWRITE | G_PARAM_CONSTRUCT | G_PARAM_STATIC_STRINGS;
 
     g_object_class_install_property (gobject_class,
                                      PROP_REMEMBER_LAST_WINDOW_SIZE,
@@ -301,6 +329,23 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      0, G_MAXINT, 0,
                                      flags));
 
+    /**
+    * MidoriWebSettings:last-window-state:
+    *
+    * The last saved window state.
+    *
+    * Since: 0.1.3
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_LAST_WINDOW_STATE,
+                                     g_param_spec_enum (
+                                     "last-window-state",
+                                     "Last window state",
+                                     "The last saved window state",
+                                     MIDORI_TYPE_WINDOW_STATE,
+                                     MIDORI_WINDOW_NORMAL,
+                                     flags));
+
     g_object_class_install_property (gobject_class,
                                      PROP_LAST_PANEL_POSITION,
                                      g_param_spec_int (
@@ -386,13 +431,29 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      MIDORI_TOOLBAR_DEFAULT,
                                      flags));
 
+    /**
+    * MidoriWebSettings:progress-in-location:
+    *
+    * Whether to show loading progress in the location entry.
+    *
+    * Since: 0.1.3
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_PROGRESS_IN_LOCATION,
+                                     g_param_spec_boolean (
+                                     "progress-in-location",
+                                     _("Show progress in location entry"),
+                                     _("Whether to show loading progress in the location entry"),
+                                     TRUE,
+                                     flags));
+
     g_object_class_install_property (gobject_class,
                                      PROP_TOOLBAR_ITEMS,
                                      g_param_spec_string (
                                      "toolbar-items",
                                      _("Toolbar Items"),
                                      _("The items to show on the toolbar"),
-                                     "Back,Forward,ReloadStop,Location,Trash",
+                                     "Back,Forward,ReloadStop,Location,Panel,Trash,Search",
                                      flags));
 
     g_object_class_install_property (gobject_class,
@@ -404,6 +465,22 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      FALSE,
                                      flags));
 
+    /**
+    * MidoriWebSettings:right-sidepanel:
+    *
+    * Whether to align the sidepanel on the right.
+    *
+    * Since: 0.1.3
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_RIGHT_ALIGN_SIDEPANEL,
+                                     g_param_spec_boolean (
+                                     "right-align-sidepanel",
+                                     _("Align sidepanel on the right"),
+                                     _("Whether to align the sidepanel on the right"),
+                                     FALSE,
+                                     flags));
+
 
     g_object_class_install_property (gobject_class,
                                      PROP_LOAD_ON_STARTUP,
@@ -514,8 +591,8 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Where to open new pages"),
                                      MIDORI_TYPE_NEW_PAGE,
                                      MIDORI_NEW_PAGE_TAB,
-    g_signal_lookup ("create-web-view", WEBKIT_TYPE_WEB_VIEW)
-        ? G_PARAM_READWRITE : G_PARAM_READABLE));
+    (g_signal_lookup ("create-web-view", WEBKIT_TYPE_WEB_VIEW)
+        ? G_PARAM_READWRITE : G_PARAM_READABLE) | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_OPEN_EXTERNAL_PAGES_IN,
@@ -525,7 +602,11 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Where to open externally opened pages"),
                                      MIDORI_TYPE_NEW_PAGE,
                                      MIDORI_NEW_PAGE_TAB,
-                                     G_PARAM_READWRITE));
+    #if HAVE_UNIQUE
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+    #else
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+    #endif
 
     g_object_class_install_property (gobject_class,
                                      PROP_MIDDLE_CLICK_OPENS_SELECTION,
@@ -561,7 +642,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Open popups in tabs"),
                                      _("Whether to open popup windows in tabs"),
                                      TRUE,
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 
     if (!g_object_class_find_property (gobject_class, "enforce-96-dpi"))
@@ -579,7 +660,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      "Enforce 96 DPI",
                                      "Whether to enforce a resolution of 96 DPI",
                                      FALSE,
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
     if (!g_object_class_find_property (gobject_class, "enable-developer-extras"))
     /**
@@ -596,7 +677,26 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      "Enable Developer Extras",
                                      "Whether to enable extra developer tools",
                                      FALSE,
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+
+    if (!g_object_class_find_property (gobject_class, "zoom-text-and-images"))
+    /**
+    * MidoriWebSettings:zoom-text-and-images:
+    *
+    * Whether to zoom text and images.
+    *
+    * Since: 0.1.3
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_ZOOM_TEXT_AND_IMAGES,
+                                     g_param_spec_boolean (
+                                     "zoom-text-and-images",
+                                     _("Zoom Text and Images"),
+                                     _("Whether to zoom text and images"),
+                                     FALSE,
+    (g_object_class_find_property (g_type_class_ref (WEBKIT_TYPE_WEB_VIEW),
+        "full-content-zoom") ? G_PARAM_READWRITE : G_PARAM_READABLE)
+        | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_ACCEPT_COOKIES,
@@ -607,9 +707,9 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      MIDORI_TYPE_ACCEPT_COOKIES,
                                      MIDORI_ACCEPT_COOKIES_ALL,
                                      #if HAVE_LIBSOUP
-                                     G_PARAM_READWRITE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
                                      #else
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
                                      #endif
 
     g_object_class_install_property (gobject_class,
@@ -619,7 +719,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Original cookies only"),
                                      _("Accept cookies from the original website only"),
                                      FALSE,
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_MAXIMUM_COOKIE_AGE,
@@ -628,7 +728,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Maximum cookie age"),
                                      _("The maximum number of days to save cookies for"),
                                      0, G_MAXINT, 30,
-                                     G_PARAM_READWRITE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
 
 
@@ -657,7 +757,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Remember last form inputs"),
                                      _("Whether the last form inputs are saved"),
                                      TRUE,
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
     g_object_class_install_property (gobject_class,
                                      PROP_REMEMBER_LAST_DOWNLOADED_FILES,
@@ -666,7 +766,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Remember last downloaded files"),
                                      _("Whether the last downloaded files are saved"),
                                      TRUE,
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 
 
 
@@ -678,9 +778,29 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("The proxy server used for HTTP connections"),
                                      NULL,
                                      #if HAVE_LIBSOUP
-                                     G_PARAM_READWRITE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
                                      #else
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+                                     #endif
+
+    /**
+    * MidoriWebSettings:auto-detect-proxy:
+    *
+    * Whether to detect the proxy server automatically from the environment
+    *
+    * Since: 0.1.3
+    */
+    g_object_class_install_property (gobject_class,
+                                     PROP_AUTO_DETECT_PROXY,
+                                     g_param_spec_boolean (
+                                     "auto-detect-proxy",
+                                     _("Detect proxy server automatically"),
+        _("Whether to detect the proxy server automatically from the environment"),
+                                     TRUE,
+                                     #if HAVE_LIBSOUP
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+                                     #else
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
                                      #endif
 
     /**
@@ -700,9 +820,9 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      MIDORI_TYPE_IDENTITY,
                                      MIDORI_IDENT_MIDORI,
                                      #if HAVE_LIBSOUP
-                                     G_PARAM_READWRITE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
                                      #else
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
                                      #endif
 
     /**
@@ -720,9 +840,9 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("The application identification string"),
                                      NULL,
                                      #if HAVE_LIBSOUP
-                                     G_PARAM_READWRITE));
+                                     G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
                                      #else
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
                                      #endif
 
     g_object_class_install_property (gobject_class,
@@ -732,7 +852,7 @@ midori_web_settings_class_init (MidoriWebSettingsClass* class)
                                      _("Cache size"),
                                      _("The allowed size of the cache"),
                                      0, G_MAXINT, 100,
-                                     G_PARAM_READABLE));
+                                     G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
 }
 
 static void
@@ -766,6 +886,10 @@ static void
 midori_web_settings_init (MidoriWebSettings* web_settings)
 {
     web_settings->http_proxy = NULL;
+    web_settings->open_popups_in_tabs = TRUE;
+    web_settings->remember_last_form_inputs = TRUE;
+    web_settings->remember_last_downloaded_files = TRUE;
+    web_settings->auto_detect_proxy = TRUE;
 
     g_signal_connect (web_settings, "notify::default-encoding",
                       G_CALLBACK (notify_default_encoding_cb), NULL);
@@ -862,6 +986,9 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_LAST_WINDOW_HEIGHT:
         web_settings->last_window_height = g_value_get_int (value);
         break;
+    case PROP_LAST_WINDOW_STATE:
+        web_settings->last_window_state = g_value_get_enum (value);
+        break;
     case PROP_LAST_PANEL_POSITION:
         web_settings->last_panel_position = g_value_get_int (value);
         break;
@@ -891,12 +1018,18 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_TOOLBAR_STYLE:
         web_settings->toolbar_style = g_value_get_enum (value);
         break;
+    case PROP_PROGRESS_IN_LOCATION:
+        web_settings->progress_in_location = g_value_get_boolean (value);
+        break;
     case PROP_TOOLBAR_ITEMS:
         katze_assign (web_settings->toolbar_items, g_value_dup_string (value));
         break;
     case PROP_COMPACT_SIDEPANEL:
         web_settings->compact_sidepanel = g_value_get_boolean (value);
         break;
+    case PROP_RIGHT_ALIGN_SIDEPANEL:
+        web_settings->right_align_sidepanel = g_value_get_boolean (value);
+        break;
 
     case PROP_LOAD_ON_STARTUP:
         web_settings->load_on_startup = g_value_get_enum (value);
@@ -968,6 +1101,9 @@ midori_web_settings_set_property (GObject*      object,
         web_settings->open_popups_in_tabs = g_value_get_boolean (value);
         break;
 
+    case PROP_ZOOM_TEXT_AND_IMAGES:
+        web_settings->zoom_text_and_images = g_value_get_boolean (value);
+        break;
     case PROP_ACCEPT_COOKIES:
         web_settings->accept_cookies = g_value_get_enum (value);
         break;
@@ -994,6 +1130,9 @@ midori_web_settings_set_property (GObject*      object,
     case PROP_HTTP_PROXY:
         katze_assign (web_settings->http_proxy, g_value_dup_string (value));
         break;
+    case PROP_AUTO_DETECT_PROXY:
+        web_settings->auto_detect_proxy = g_value_get_boolean (value);
+        break;
     case PROP_IDENTIFY_AS:
         web_settings->identify_as = g_value_get_enum (value);
         if (web_settings->identify_as != MIDORI_IDENT_CUSTOM)
@@ -1034,6 +1173,9 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_LAST_WINDOW_HEIGHT:
         g_value_set_int (value, web_settings->last_window_height);
         break;
+    case PROP_LAST_WINDOW_STATE:
+        g_value_set_enum (value, web_settings->last_window_state);
+        break;
     case PROP_LAST_PANEL_POSITION:
         g_value_set_int (value, web_settings->last_panel_position);
         break;
@@ -1063,12 +1205,18 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_TOOLBAR_STYLE:
         g_value_set_enum (value, web_settings->toolbar_style);
         break;
+    case PROP_PROGRESS_IN_LOCATION:
+        g_value_set_boolean (value, web_settings->progress_in_location);
+        break;
     case PROP_TOOLBAR_ITEMS:
         g_value_set_string (value, web_settings->toolbar_items);
         break;
     case PROP_COMPACT_SIDEPANEL:
         g_value_set_boolean (value, web_settings->compact_sidepanel);
         break;
+    case PROP_RIGHT_ALIGN_SIDEPANEL:
+        g_value_set_boolean (value, web_settings->right_align_sidepanel);
+        break;
 
     case PROP_LOAD_ON_STARTUP:
         g_value_set_enum (value, web_settings->load_on_startup);
@@ -1126,6 +1274,9 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_ENABLE_DEVELOPER_EXTRAS:
         g_value_set_boolean (value, FALSE);
         break;
+    case PROP_ZOOM_TEXT_AND_IMAGES:
+        g_value_set_boolean (value, web_settings->zoom_text_and_images);
+        break;
     case PROP_ACCEPT_COOKIES:
         g_value_set_enum (value, web_settings->accept_cookies);
         break;
@@ -1152,6 +1303,9 @@ midori_web_settings_get_property (GObject*    object,
     case PROP_HTTP_PROXY:
         g_value_set_string (value, web_settings->http_proxy);
         break;
+    case PROP_AUTO_DETECT_PROXY:
+        g_value_set_boolean (value, web_settings->auto_detect_proxy);
+        break;
     case PROP_IDENTIFY_AS:
         g_value_set_enum (value, web_settings->identify_as);
         break;
