@@ -1,5 +1,5 @@
 /*
- Copyright (C) 2007-2008 Christian Dywan <christian@twotoasts.de>
+ Copyright (C) 2007-2009 Christian Dywan <christian@twotoasts.de>
 
  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
@@ -10,6 +10,7 @@
 */
 
 #include "sokoke.h"
+#include "midori-stock.h"
 
 #if HAVE_CONFIG_H
     #include <config.h>
@@ -18,12 +19,19 @@
 #if HAVE_UNISTD_H
     #include <unistd.h>
 #endif
+#include <stdlib.h>
 #include <string.h>
 
 #include <gdk/gdkkeysyms.h>
 #include <glib/gi18n.h>
 #include <glib/gprintf.h>
 
+#if HAVE_LIBIDN
+    #include <stringprep.h>
+    #include <punycode.h>
+    #include <idna.h>
+#endif
+
 static gchar*
 sokoke_js_string_utf8 (JSStringRef js_string)
 {
@@ -120,6 +128,46 @@ sokoke_spawn_program (const gchar* command,
     return TRUE;
 }
 
+static gchar*
+sokoke_idn_to_punycode (gchar* uri)
+{
+    #if HAVE_LIBIDN
+    gchar* hostname;
+    char *s;
+    uint32_t *q;
+    int rc;
+    gchar *result;
+
+    if (g_str_has_prefix (uri, "http://"))
+        hostname = &uri[7];
+    else if (g_str_has_prefix (uri, "https://"))
+        hostname = &uri[8];
+    else
+        hostname = uri;
+
+    if (!(q = stringprep_utf8_to_ucs4 (hostname, -1, NULL)))
+        return uri;
+
+    rc = idna_to_ascii_4z (q, &s, IDNA_ALLOW_UNASSIGNED);
+    free (q);
+    if (rc != IDNA_SUCCESS)
+        return uri;
+
+    if (g_str_has_prefix (uri, "http://"))
+        result = g_strdup_printf ("http://%s", s);
+    else if (g_str_has_prefix (uri, "https://"))
+        result = g_strdup_printf ("https://%s", s);
+    else
+        result = g_strdup (s);
+    g_free (uri);
+    free (s);
+
+    return result;
+    #else
+    return uri;
+    #endif
+}
+
 gchar*
 sokoke_magic_uri (const gchar* uri,
                   KatzeArray*  search_engines)
@@ -142,8 +190,7 @@ sokoke_magic_uri (const gchar* uri,
     if (g_path_is_absolute (uri))
         return g_strconcat ("file://", uri, NULL);
     /* Construct an absolute path if the file is relative */
-    if (g_file_test (uri, G_FILE_TEST_EXISTS)
-        && g_file_test (uri, G_FILE_TEST_IS_REGULAR))
+    if (g_file_test (uri, G_FILE_TEST_EXISTS | G_FILE_TEST_IS_REGULAR))
     {
         current_dir = g_get_current_dir ();
         result = g_strconcat ("file://", current_dir,
@@ -156,9 +203,10 @@ sokoke_magic_uri (const gchar* uri,
     {
         /* Do we have a domain, ip address or localhost? */
         search = strchr (uri, ':');
-        if (search && search[0] && !g_ascii_isalpha (search[1]))
+        if (search && search[0] &&
+            !g_ascii_isalpha (search[1]) && search[1] != ' ')
             if (!strchr (search, '.'))
-                return g_strconcat ("http://", uri, NULL);
+                return sokoke_idn_to_punycode (g_strconcat ("http://", uri, NULL));
         if (!strcmp (uri, "localhost") || g_str_has_prefix (uri, "localhost/"))
             return g_strconcat ("http://", uri, NULL);
         parts = g_strsplit (uri, ".", 0);
@@ -170,7 +218,7 @@ sokoke_magic_uri (const gchar* uri,
                     search = g_strconcat ("http://", uri, NULL);
             g_free (parts);
             if (search)
-                return search;
+                return sokoke_idn_to_punycode (search);
         }
         /* We don't want to search? So return early. */
         if (!search_engines)
@@ -195,7 +243,7 @@ sokoke_magic_uri (const gchar* uri,
         }
         return search;
     }
-    return g_strdup (uri);
+    return sokoke_idn_to_punycode (g_strdup (uri));
 }
 
 void
@@ -631,65 +679,90 @@ sokoke_image_menu_item_new_ellipsized (const gchar* label)
 }
 
 /**
- * sokoke_tree_view_get_selected_iter:
- * @tree_view: a #GtkTreeView
- * @model: a pointer to store the model, or %NULL
- * @iter: a pointer to store the iter, or %NULL
+ * sokoke_time_t_to_julian:
+ * @timestamp: a time_t timestamp value
  *
- * Determines whether there is a selection in the tree view
- * and sets the @iter to the current selection.
- *
- * If there is a selection and @model is not %NULL, it is
- * set to the model, mainly for convenience.
- *
- * Either @model or @iter or both can be %NULL in which case
- * no value will be assigned in any case.
- *
- * Return value: %TRUE if there is a selection
- **/
-gboolean
-sokoke_tree_view_get_selected_iter (GtkTreeView*   tree_view,
-                                    GtkTreeModel** model,
-                                    GtkTreeIter*   iter)
-{
-    GtkTreeSelection* selection;
-
-    g_return_val_if_fail (GTK_IS_TREE_VIEW (tree_view), FALSE);
-
-    selection = gtk_tree_view_get_selection (tree_view);
-    if (selection)
-        if (gtk_tree_selection_get_selected (selection, model, iter))
-            return TRUE;
-    return FALSE;
-}
-
-/**
- * sokoke_days_between:
- * @day1: a time_t timestamp value
- * @day2: a time_t timestamp value
- *
- * Calculates the number of days between two timestamps.
+ * Calculates a unix timestamp to a julian day value.
  *
  * Return value: an integer.
  **/
-gint
-sokoke_days_between (const time_t* day1,
-                     const time_t* day2)
+gint64
+sokoke_time_t_to_julian (const time_t* timestamp)
 {
-    GDate* date1;
-    GDate* date2;
-    gint age;
+    GDate* date;
+    gint64 julian;
 
-    date1 = g_date_new ();
-    date2 = g_date_new ();
+    date = g_date_new ();
 
-    g_date_set_time_t (date1, *day1);
-    g_date_set_time_t (date2, *day2);
+    g_date_set_time_t (date, *timestamp);
+    julian = (gint64)g_date_get_julian (date);
 
-    age = g_date_days_between (date1, date2);
+    g_date_free (date);
 
-    g_date_free (date1);
-    g_date_free (date2);
+    return julian;
+}
 
-    return age;
+/**
+ * sokoke_register_stock_items:
+ *
+ * Registers several custom stock items used throughout Midori.
+ **/
+void
+sokoke_register_stock_items (void)
+{
+    typedef struct
+    {
+        const gchar* stock_id;
+        const gchar* label;
+        GdkModifierType modifier;
+        guint keyval;
+        const gchar* fallback;
+    } FatStockItem;
+    GtkIconSource* icon_source;
+    GtkIconSet* icon_set;
+    GtkIconFactory* factory = gtk_icon_factory_new ();
+    gsize i;
+
+    static FatStockItem items[] =
+    {
+        { STOCK_EXTENSION, NULL, 0, 0, GTK_STOCK_CONVERT },
+        { STOCK_NEWS_FEED, NULL, 0, 0, GTK_STOCK_INDEX },
+        { STOCK_SCRIPT, NULL, 0, 0, GTK_STOCK_EXECUTE },
+        { STOCK_STYLE, NULL, 0, 0, GTK_STOCK_SELECT_COLOR },
+        { STOCK_TRANSFER, NULL, 0, 0, GTK_STOCK_SAVE },
+
+        { STOCK_BOOKMARK,       N_("_Bookmark"), 0, 0, GTK_STOCK_FILE },
+        { STOCK_BOOKMARKS,      N_("_Bookmarks"), 0, 0, GTK_STOCK_DIRECTORY },
+        { STOCK_BOOKMARK_ADD,   N_("_Add Bookmark"), 0, 0, GTK_STOCK_ADD },
+        { STOCK_CONSOLE,        N_("_Console"), 0, 0, GTK_STOCK_DIALOG_WARNING },
+        { STOCK_EXTENSIONS,     N_("_Extensions"), 0, 0, GTK_STOCK_CONVERT },
+        { STOCK_HISTORY,        N_("_History"), 0, 0, GTK_STOCK_SORT_ASCENDING },
+        { STOCK_HOMEPAGE,       N_("_Homepage"), 0, 0, GTK_STOCK_HOME },
+        { STOCK_SCRIPTS,        N_("_Userscripts"), 0, 0, GTK_STOCK_EXECUTE },
+        { STOCK_STYLES,         N_("User_styles"), 0, 0, GTK_STOCK_SELECT_COLOR },
+        { STOCK_TAB_NEW,        N_("New _Tab"), 0, 0, GTK_STOCK_ADD },
+        { STOCK_TRANSFERS,      N_("_Transfers"), 0, 0, GTK_STOCK_SAVE },
+        { STOCK_USER_TRASH,     N_("_Closed Tabs and Windows"), 0, 0, "gtk-undo-ltr" },
+        { STOCK_WINDOW_NEW,     N_("New _Window"), 0, 0, GTK_STOCK_ADD },
+    };
+
+    for (i = 0; i < G_N_ELEMENTS (items); i++)
+    {
+        icon_set = gtk_icon_set_new ();
+        icon_source = gtk_icon_source_new ();
+        if (items[i].fallback)
+        {
+            gtk_icon_source_set_icon_name (icon_source, items[i].fallback);
+            items[i].fallback = NULL;
+            gtk_icon_set_add_source (icon_set, icon_source);
+        }
+        gtk_icon_source_set_icon_name (icon_source, items[i].stock_id);
+        gtk_icon_set_add_source (icon_set, icon_source);
+        gtk_icon_source_free (icon_source);
+        gtk_icon_factory_add (factory, items[i].stock_id, icon_set);
+        gtk_icon_set_unref (icon_set);
+    }
+    gtk_stock_add ((GtkStockItem*)items, G_N_ELEMENTS (items));
+    gtk_icon_factory_add_default (factory);
+    g_object_unref (factory);
 }
