@@ -25,13 +25,19 @@ main (int    argc,
     gchar* uri;
     gchar* a, *b;
 
-    gtk_init (&argc, &argv);
+    gtk_init_check (&argc, &argv);
 
     search_engines = katze_array_new (KATZE_TYPE_ITEM);
     item = g_object_new (KATZE_TYPE_ITEM,
                          "uri", "http://www.searchmash.com/search/%s",
                          "token", "sm", NULL);
     katze_array_add_item (search_engines, item);
+    g_object_unref (item);
+    item = g_object_new (KATZE_TYPE_ITEM,
+                         "uri", "http://www.searchmash.com/search/",
+                         "token", "se", NULL);
+    katze_array_add_item (search_engines, item);
+    g_object_unref (item);
 
 #define test_input(input, expected) \
   uri = sokoke_magic_uri (input, search_engines); \
@@ -60,10 +66,12 @@ main (int    argc,
     g_free (b);
     test_input ("localhost", "http://localhost");
     test_input ("localhost:8000", "http://localhost:8000");
+    test_input ("localhost/rss", "http://localhost/rss");
     test_input ("192.168.1.1", "http://192.168.1.1");
     test_input ("192.168.1.1:8000", "http://192.168.1.1:8000");
     test_input ("sm midori", SM "midori");
     test_input ("sm cats dogs", SM "cats dogs");
+    test_input ("se cats dogs", SM "cats dogs");
     test_input ("dict midori", NULL);
     test_input ("cats", NULL);
     test_input ("cats dogs", NULL);
@@ -83,6 +91,24 @@ main (int    argc,
     test_input ("verbose de.po", NULL);
     test_input ("g de.po verbose", NULL);
     test_input ("sm de.po verbose", SM "de.po verbose");
+    test_input ("javascript:alert(1)", "javascript:alert(1)");
+    test_input ("sm warning: configure /dev/net: virtual",
+                SM "warning: configure /dev/net: virtual");
+    #if HAVE_LIBIDN
+    test_input ("http://www.münchhausen.at", "http://www.xn--mnchhausen-9db.at");
+    test_input ("http://www.خداوند.com/", "http://www.xn--mgbndb8il.com/");
+    test_input ("айкидо.com", "http://xn--80aildf0a.com");
+    test_input ("http://東京理科大学.jp", "http://xn--1lq68wkwbj6ugkpigi.jp");
+    test_input ("https://青のネコ",  "https://xn--u9jthzcs263c");
+    #else
+    test_input ("http://www.münchhausen.at", "http://www.münchhausen.at");
+    test_input ("http://www.خداوند.com/", "http://www.خداوند.com/");
+    test_input ("айкидо.com", "http://айкидо.com");
+    test_input ("http://東京理科大学.jp", "http://東京理科大学.jp");
+    test_input ("https://青のネコ.co.jp",  "https://青のネコ.co.jp");
+    #endif
+    test_input ("sm Küchenzubehör", SM "Küchenzubehör");
+    test_input ("sm 東京理科大学", SM "東京理科大学");
 
     return 0;
 }
